@@ -29,6 +29,7 @@ extern "C" {
 
 extern "C" {
 #include "muesli.h"
+#include "muesli-internal.h"
 }
 
 // We've defined these in our config.h; clear them to stop warnings
@@ -461,38 +462,38 @@ octave_evaluator_init(evaluator_interface *interface)
   interface->state = our_octave_state;
   octave_interface = interface;
 
-  // Extend system
-
-  // Fill in: register all these language extensions (as applicable):
-  // Muesli_Add_Fn_1(interface, (char*)"get_parameter", octave_get_parameter);
-  // Muesli_Add_Fn_2(interface, (char*)"set_parameter", octave_set_parameter);
-  // Muesli_Add_Fn_0(interface, (char*)"evolution_parameters", octave_get_parameters);
-  // Muesli_Add_Fn_1(interface, (char*)"modify_evolution_parameters", octave_set_parameters);
-
-  XDEFUN_INTERNAL (custom, args, , false, "-*- texinfo -*-\n\
+  if (muesli_extensions & MUESLI_EXTENSION_CUSTOM) {
+    XDEFUN_INTERNAL (custom, args, , false, "-*- texinfo -*-\n\
 @deftypefn {Extended Built-in Function} {} custom (@var{x})\n\
 Evaluate @var{x} using the custom evaluator.\n\
 @end deftypefn");
-  XDEFUN_INTERNAL (eval_in_language, args, , false, "-*- texinfo -*-\n\
+  }
+  if (muesli_extensions & MUESLI_EXTENSION_EVAL_IN_LANGUAGE) {
+    XDEFUN_INTERNAL (eval_in_language, args, , false, "-*- texinfo -*-\n\
 @deftypefn {Extended Built-in Function} {} eval_in_language (@var{language}) @var{fragment}\n\
 Evaluate @var{fragment} using the evaluator for @var{language}.\n\
 @end deftypefn");
-  XDEFUN_INTERNAL(set_parameter, args, , false, "-*- texinfo -*-\n\
+  }
+  if (muesli_extensions & MUESLI_EXTENSION_SINGLE_PARAMETER) {
+    XDEFUN_INTERNAL(set_parameter, args, , false, "-*- texinfo -*-\n\
 @deftypefn {Extended Built-in Function} {} set_parameter(@var{name} @var{value})\n\
 Set parameter @var{name} to @var{value}.\n\
 @end deftypefn");
-  XDEFUN_INTERNAL(get_parameter, args, , false, "-*- texinfo -*-\n\
+    XDEFUN_INTERNAL(get_parameter, args, , false, "-*- texinfo -*-\n\
 @deftypefn {Extended Built-in Function} {} get_parameter(@var{name})\n\
 Get parameter @var{name}.\n\
 @end deftypefn");
-  XDEFUN_INTERNAL(set_parameters, args, , false, "-*- texinfo -*-\n\
+  }
+  if (muesli_extensions & MUESLI_EXTENSION_PARAMETER_BLOCK) {
+    XDEFUN_INTERNAL(set_parameters, args, , false, "-*- texinfo -*-\n\
 @deftypefn {Extended Built-in Function} {} set_parameters(@var{map})\n\
 Set parameters mentioned in @var{map}.\n\
 @end deftypefn");
-  XDEFUN_INTERNAL(get_parameters, args, , false, "-*- texinfo -*-\n\
+    XDEFUN_INTERNAL(get_parameters, args, , false, "-*- texinfo -*-\n\
 @deftypefn {Extended Built-in Function} {} get_parameters()\n\
 Return current parameters as a map.\n\
 @end deftypefn");
+  }
   
   // Set up option names
   Octave_map option_names_table;
