@@ -23,6 +23,7 @@
 
 #include "../config.h"
 #include "muesli.h"
+#include "muesli-internal.h"
 
 #if defined(HAVE_LIBPYTHON) && defined(HAVE_PYTHON2_6_PYTHON_H)
 
@@ -240,9 +241,6 @@ python_evaluator_init(evaluator_interface *interface)
   // register all these language extensions (as applicable):
   Py_InitModule("Muesli", MuesliMethods);
 
-#if 1
-  fprintf(stderr, "Initialized Python\n");
-#endif
 
 #if 0
   // Set up option names
