@@ -26,6 +26,7 @@
 #ifdef HAVE_LIBULSIOD
 
 #include "muesli.h"
+#include "muesli-internal.h"
 // #include "lang-extns.h"
 
 #include <limits.h>
@@ -152,39 +153,12 @@ muesli_siod_get_parameters()
   while (names != NIL) {
     if (CONSP(names)) {
       siod_LISP pair = siod_car(names);
-      siod_LISP lisp_result;
-
-      muesli_value_t result = (current_interface->handle_option)(current_interface->app_params,
+      siod_LISP lisp_result = muesli_to_siod((current_interface->handle_option)(current_interface->app_params,
 								 (int)(FLONM(siod_cdr(pair))), // opt
 								 NULL, 0.0,		     // value
 								 0,		     // set
-								 (const char *)"siod");
+										(const char *)"siod"));
 
-      switch (result.type) {
-      case muesli_value_string:
-	if (result.data.as_string != NULL) {
-	  lisp_result = siod_strcons(strlen(result.data.as_string), result.data.as_string);
-	} else {
-	  lisp_result = NIL;
-	}
-	break;
-      case muesli_value_float:
-	lisp_result = siod_flocons(result.data.as_float);
-	break;
-      case muesli_value_integer:
-	lisp_result = siod_flocons((float)result.data.as_int);
-	break;
-      case muesli_value_boolean:
-	if (result.data.as_int) {
-	  lisp_result = sym_t;
-	} else {
-	  lisp_result = NIL;
-	}
-	break;
-      default:
-	lisp_result = NIL;
-	break;
-      }
       if (lisp_result != NIL) {
 	results = siod_cons(siod_cons(siod_car(pair), lisp_result), results);
       }
@@ -212,28 +186,9 @@ muesli_siod_custom_eval_function(siod_LISP eval_in)
       return eval_in;
     } else {
       char *str = get_c_string(eval_in);
-      muesli_value_t result = (custom_evaluator->eval_string)(custom_evaluator,
-							      str, strlen(str),
-							      ambient_transient);
-      switch (result.type) {
-      case muesli_value_float:
-	return siod_flocons(result.data.as_float);
-      case muesli_value_integer:
-	return siod_flocons(result.data.as_int);
-      case muesli_value_string:
-	return siod_strcons(strlen(result.data.as_string), result.data.as_string);
-      case muesli_value_boolean:
-	if (result.data.as_int) {
-	  return sym_t;
-	} else {
-	  return NIL;
-	}
-      case muesli_value_none:
-	return NIL;
-      case muesli_value_error:
-	// todo: should probably throw something
-	return NIL;
-      }
+      return muesli_to_siod((custom_evaluator->eval_string)(custom_evaluator,
+							    str, strlen(str),
+							    ambient_transient));
     }
   } else {
 #if 0
@@ -645,7 +600,9 @@ siod_setup(evaluator_interface *new_siod_interface)
   current_interface->add_function = siod_add_function;
   current_interface->load_file = siod_load_file;
   current_interface->repl = siod_muesli_repl;
+  current_interface->array_opener = (char*)"#( ";
   current_interface->array_separator = (char*)" ";
+  current_interface->array_closer = (char*)" )";
 
   current_interface->version = siod_version();
 }
