@@ -38,11 +38,16 @@
 #endif
 
 #include "muesli.h"
+#include "muesli-internal.h"
 
 const char *muesli_version = PACKAGE_VERSION;
 
 static void muesli_default_close_evaluator(struct evaluator_interface *myself);
 
+static int muesli_app_argc = 0;
+static char **muesli_app_argv = NULL;
+static char **muesli_app_env = NULL;
+
 /***********************/
 /* Evaluator interface */
 /***********************/
@@ -50,7 +55,8 @@ static void muesli_default_close_evaluator(struct evaluator_interface *myself);
 static evaluator_interface *evaluator_interfaces = NULL;
 
 evaluator_interface *
-muesli_make_evaluator_interface(const char* new_name,
+muesli_make_evaluator_interface(const char* new_language_name,
+				const char* new_implementation_name,
 				const char *new_extension,
 				int new_is_binary,
 				void (*setup_func)(evaluator_interface *myself),
@@ -60,9 +66,14 @@ muesli_make_evaluator_interface(const char* new_name,
 				void* params,
 				void *data)
 {
-  evaluator_interface *interface = (evaluator_interface*)malloc(sizeof(evaluator_interface));
+  evaluator_interface *interface
+    = (evaluator_interface*)malloc(sizeof(evaluator_interface));
 
-  interface->name = muesli_malloc_copy_string(new_name);
+  interface->language_name = muesli_malloc_copy_string(new_language_name);
+  interface->implementation_name
+    = muesli_malloc_copy_string((new_implementation_name != NULL)
+				? new_implementation_name
+				: new_language_name);
   interface->next = NULL;
   interface->extension = muesli_malloc_copy_string(new_extension);
   interface->binary = new_is_binary;
@@ -75,11 +86,13 @@ muesli_make_evaluator_interface(const char* new_name,
   interface->load_file = NULL;
   interface->to_muesli_value = NULL;
   interface->from_muesli_value = NULL;
-  
+
   interface->handle_option = new_option_handler;
   interface->getopt_options = new_getopt_options;
   interface->repl = NULL;
+  interface->array_opener = (char*)"[ ";
   interface->array_separator = (char*)", ";
+  interface->array_closer = (char*)" ]";
 
   interface->language_verbosity = 0;
   interface->flags = 0;
@@ -93,6 +106,10 @@ muesli_make_evaluator_interface(const char* new_name,
   interface->app_params = params;
   interface->app_data = data;
 
+  interface->app_argc = muesli_app_argc;
+  interface->app_argv = muesli_app_argv;
+  interface->app_env = muesli_app_env;
+
   interface->eval_clear = NULL;
   interface->eval_give = NULL;
   interface->eval_given = NULL;
@@ -107,13 +124,21 @@ muesli_make_evaluator_interface(const char* new_name,
 
 evaluator_interface *
 muesli_copy_evaluator_interface(evaluator_interface *original,
-				const char *new_name)
+				const char *new_language_name,
+				const char *new_implementation_name)
 {
   evaluator_interface *interface = (evaluator_interface*)malloc(sizeof(evaluator_interface));
 
-  interface->name = muesli_malloc_copy_string((new_name != NULL)
-					      ? new_name
-					      : original->name);
+  interface->language_name = muesli_malloc_copy_string((new_language_name != NULL)
+						       ? new_language_name
+						       : original->language_name);
+  interface->implementation_name = muesli_malloc_copy_string((new_implementation_name != NULL)
+							     ? new_implementation_name
+							     : ((new_language_name != NULL)
+								? new_language_name
+								: ((original->implementation_name != NULL)
+								   ? original->implementation_name
+								   : original->language_name)));
   interface->next = NULL;
   interface->extension = original->extension;
   interface->binary = original->binary;
@@ -139,6 +164,10 @@ muesli_copy_evaluator_interface(evaluator_interface *original,
   interface->app_params = original->app_params;
   interface->app_data = original->app_data;
 
+  interface->app_argc = original->app_argc;
+  interface->app_argv = original->app_argv;
+  interface->app_env = original->app_env;
+
   interface->eval_clear = original->eval_clear;
   interface->eval_give = original->eval_give;
   interface->eval_given = original->eval_given;
@@ -155,14 +184,20 @@ static void
 muesli_default_close_evaluator(struct evaluator_interface *myself)
 {
   if (myself->language_verbosity > 0) {
-    fprintf(stderr, "Closing %s evaluator\n", myself->name);
+    fprintf(stderr, "Closing %s evaluator\n", myself->implementation_name);
   }
 }
 
 char*
 muesli_getName(evaluator_interface *interface)
 {
-  return interface->name;
+  return interface->implementation_name;
+}
+
+char*
+muesli_getLanguageName(evaluator_interface *interface)
+{
+  return interface->language_name;
 }
 
 int
@@ -176,10 +211,10 @@ muesli_initialize_evaluator(evaluator_interface *interface)
 {
   if (!interface->initialized) {
     if (interface->language_verbosity > 0) {
-      fprintf(stderr, "Muesli initializing %s\n", interface->name);
+      fprintf(stderr, "Muesli initializing %s\n", interface->implementation_name);
     }
     (interface->init_evaluator)(interface);
-    muesli_app_specialize(interface->name, interface);
+    muesli_app_specialize(interface->implementation_name, interface);
     interface->initialized = 1;
   }
 }
@@ -202,10 +237,10 @@ muesli_get_evaluator_by_extension(const char *extension)
     if ((evaluator->extension != NULL)
 	&& strcmp(evaluator->extension, extension) == 0) {
       if (choice_1 == NULL) {
-	fprintf(stderr, "First possibility for %s is %s\n", extension, evaluator->name);
+	fprintf(stderr, "First possibility for %s is %s\n", extension, evaluator->implementation_name);
 	choice_1 = evaluator;
       } else  {
-	fprintf(stderr, "Other possibility for %s is %s\n", extension, evaluator->name);
+	fprintf(stderr, "Other possibility for %s is %s\n", extension, evaluator->implementation_name);
 	choice_2 = evaluator;
       }
     }
@@ -217,7 +252,7 @@ muesli_get_evaluator_by_extension(const char *extension)
     if (choice_2 != NULL) {
       char *preferences = getenv("MUESLI_PREFERENCES");
       if (preferences != NULL) {
-	char *name_2 = choice_2->name;
+	char *name_2 = choice_2->implementation_name;
 	unsigned int name_2_len = strlen(name_2);
 
 
@@ -246,7 +281,7 @@ muesli_get_language_by_extension(const char *extension)
 {
   evaluator_interface *evaluator = muesli_get_evaluator_by_extension(extension);
 
-  return (evaluator != NULL) ? evaluator->name : NULL;
+  return (evaluator != NULL) ? evaluator->implementation_name : NULL;
 }
 
 int
@@ -258,7 +293,8 @@ muesli_evaluator_is_defined(const char *evaluator_name)
        evaluator != NULL;
        evaluator = evaluator->next) {
     if ((evaluator->extension != NULL)
-	&& strcmp(evaluator->name, evaluator_name) == 0) {
+	&& ((strcmp(evaluator->language_name, evaluator_name) == 0)
+	    || (strcmp(evaluator->implementation_name, evaluator_name) == 0))) {
       return 1;
     }
   }
@@ -277,8 +313,10 @@ muesli_get_named_evaluator(const char *evaluator_name, int exit_if_not_found)
   for (evaluator = evaluator_interfaces;
        evaluator != NULL;
        evaluator = evaluator->next) {
-    if ((evaluator->name != NULL)
-	&& strcmp(evaluator->name, evaluator_name) == 0) {
+    if (((evaluator->implementation_name != NULL)
+	 && (strcmp(evaluator->implementation_name, evaluator_name) == 0))
+	|| ((evaluator->language_name != NULL)
+	    && (strcmp(evaluator->language_name, evaluator_name) == 0))) {
       break;
     }
   }
@@ -374,9 +412,9 @@ muesli_print_interpreter_names(FILE *stream,
   for (eval = evaluator_interfaces;
        eval != NULL;
        eval = eval->next) {
-    fprintf(stream, format, eval->name,
+    fprintf(stream, format, eval->language_name, eval->implementation_name,
 	   (((selected_name != NULL)
-	     && (strcmp(eval->name, selected_name) == 0))
+	     && (strcmp(eval->implementation_name, selected_name) == 0))
 	    ? (selected_flag ? selected_flag : " (selected)") : ""),
 	   eval->version);
   }
@@ -390,26 +428,32 @@ evaluator_interface *no_evaluator = NULL; // might put in a novelty one to print
 
 evaluator_interface *scripting_evaluator = NULL;
 
-evaluator_interface *evaluator_R = NULL;
 evaluator_interface *evaluator_common_lisp = NULL;
 evaluator_interface *evaluator_custom = NULL;
+evaluator_interface *evaluator_ecl = NULL;
+evaluator_interface *evaluator_elk = NULL;
 evaluator_interface *evaluator_erlang = NULL;
 evaluator_interface *evaluator_external_exec = NULL;
 evaluator_interface *evaluator_external_pipe = NULL;
+evaluator_interface *evaluator_gs = NULL;
 evaluator_interface *evaluator_guile = NULL;
 evaluator_interface *evaluator_haskell = NULL;
 evaluator_interface *evaluator_javascript = NULL;
 evaluator_interface *evaluator_jvm = NULL;
+evaluator_interface *evaluator_ksi = NULL;
 evaluator_interface *evaluator_lua = NULL;
 evaluator_interface *evaluator_machine = NULL;
+evaluator_interface *evaluator_matheval = NULL;
 evaluator_interface *evaluator_octave = NULL;
 evaluator_interface *evaluator_parrot = NULL;
 evaluator_interface *evaluator_perl = NULL;
 evaluator_interface *evaluator_prolog = NULL;
 evaluator_interface *evaluator_python = NULL;
+evaluator_interface *evaluator_R = NULL;
 evaluator_interface *evaluator_ruby = NULL;
 evaluator_interface *evaluator_siod = NULL;
 evaluator_interface *evaluator_slang = NULL;
+evaluator_interface *evaluator_smalltalk = NULL;
 evaluator_interface *evaluator_stack_code = NULL;
 evaluator_interface *evaluator_tcc = NULL;
 evaluator_interface *evaluator_tcl = NULL;
@@ -430,8 +474,8 @@ muesli_set_evaluator_app_specializer(char *evaluator_name,
   for (evaluator = evaluator_interfaces;
        evaluator != NULL;
        evaluator = evaluator->next) {
-    if ((evaluator->name != NULL)
-	&& strcmp(evaluator->name, evaluator_name) == 0) {
+    if ((evaluator->implementation_name != NULL)
+	&& strcmp(evaluator->implementation_name, evaluator_name) == 0) {
       break;
     }
   }
@@ -455,7 +499,8 @@ muesli_app_specialize(char *name,
 
 // A wrapper for new evaluator_interface, that registers the name in the map evaluators_by_name
 evaluator_interface *
-muesli_define_evaluator(const char *new_name,
+muesli_define_evaluator(const char *new_language_name,
+			const char *new_implementation_name,
 			const char *new_extension,
 			int new_is_binary,
 			void (*new_setup)(evaluator_interface *myself),
@@ -465,17 +510,21 @@ muesli_define_evaluator(const char *new_name,
 			void *new_params,
 			void *new_data)
 {
-  evaluator_interface *interface = muesli_make_evaluator_interface(new_name, new_extension, new_is_binary,
-								   new_setup, new_init,
-								   new_option_handler, new_getopt_options,
-								   new_params, new_data);
+  evaluator_interface *interface
+    = muesli_make_evaluator_interface(new_language_name,
+				      new_implementation_name,
+				      new_extension,
+				      new_is_binary,
+				      new_setup, new_init,
+				      new_option_handler, new_getopt_options,
+				      new_params, new_data);
 
   interface->next = evaluator_interfaces;
   evaluator_interfaces = interface;
 
   if (new_is_binary
       && (interface->eval_0 == NULL)) {
-    fprintf(stderr, "Warning: The evaluator \"%s\" claims to be binary, but cannot be\n         used as a binary fitness evaluator, as it lacks an eval_0 function\n", new_name);
+    fprintf(stderr, "Warning: The evaluator \"%s\" claims to be binary, but cannot be\n         used as a binary fitness evaluator, as it lacks an eval_0 function\n", new_implementation_name);
   }
 
   return interface;
@@ -483,7 +532,8 @@ muesli_define_evaluator(const char *new_name,
 
 evaluator_interface *
 muesli_define_variant_evaluator(evaluator_interface *original,
-				const char *new_name,
+				const char *new_language_name,
+				const char *new_implementation_name,
 				const char *new_underlying_program,
 				const char *new_underlying_startup_string,
 				const char *new_underlying_prompt_string,
@@ -491,12 +541,13 @@ muesli_define_variant_evaluator(evaluator_interface *original,
 {
   evaluator_interface *interface =
     muesli_copy_evaluator_interface(original,
-				    new_name);
+				    new_language_name,
+				    new_implementation_name);
 
-  interface->underlying_program = new_underlying_program;
-  interface->underlying_startup_string = new_underlying_startup_string;
-  interface->underlying_prompt_string = new_underlying_prompt_string;
-  interface->underlying_shutdown_command = new_underlying_shutdown_command;
+  interface->underlying_program = muesli_malloc_copy_string(new_underlying_program);
+  interface->underlying_startup_string = muesli_malloc_copy_string(new_underlying_startup_string);
+  interface->underlying_prompt_string = muesli_malloc_copy_string(new_underlying_prompt_string);
+  interface->underlying_shutdown_command = muesli_malloc_copy_string(new_underlying_shutdown_command);
 
   interface->next = evaluator_interfaces;
   evaluator_interfaces = interface;
@@ -511,155 +562,224 @@ muesli_register_evaluators(void *new_app_params,
 			   void *new_app_data,
 			   option_handler_t params_handler,
 			   struct option *getopt_options,
-			   unsigned int extensions)
+			   unsigned int extensions,
+			   int app_argc, char **app_argv,
+			   char **app_env)
 {
+  muesli_app_argc = app_argc;
+  muesli_app_argv = app_argv;
+  muesli_app_env = app_env;
+
   muesli_extensions = extensions;
   // These create the interfaces, and give them the information
   // they'll need when the interpreters initialize.  These do not
   // initialize the interpreters; that happens later.
-  evaluator_external_exec = muesli_define_evaluator("exec", "sh", 0,
+  evaluator_external_exec = muesli_define_evaluator("exec", NULL, "sh", 0,
 						    external_exec_setup, exec_evaluator_init,
 						    params_handler, getopt_options,
 						    new_app_params, new_app_data);
-  evaluator_external_pipe = muesli_define_evaluator("pipe", "", 0,
+  evaluator_external_pipe = muesli_define_evaluator("pipe", NULL, "", 0,
 						    external_pipe_setup, pipe_evaluator_init,
 						    params_handler, getopt_options,
 						    new_app_params, new_app_data);
-  evaluator_custom = muesli_define_evaluator("custom", "cus", 1,
+  evaluator_custom = muesli_define_evaluator("custom", NULL, NULL, 1,
 					     custom_setup, custom_evaluator_init,
 					     params_handler, getopt_options,
 					     new_app_params, new_app_data);
-  evaluator_zero = muesli_define_evaluator("zero", "zero", 1,
+  evaluator_zero = muesli_define_evaluator("zero", NULL, NULL, 1,
 					   zero_setup, zero_evaluator_init,
 					   params_handler, getopt_options,
 					   new_app_params, new_app_data);
-#ifdef HAVE_LIBR
-  evaluator_R = muesli_define_evaluator("R", "r", 0,
-					R_setup, R_evaluator_init,
-					params_handler, getopt_options,
-					new_app_params, new_app_data);
+#ifdef HAVE_LIBECL
+  evaluator_ecl = muesli_define_evaluator("Common Lisp", "ecl", "lisp", 0,
+					  ecl_setup, ecl_evaluator_init,
+					  params_handler, getopt_options,
+					  new_app_params, new_app_data);
+#endif
+#ifdef HAVE_LIBELK
+  evaluator_elk = muesli_define_evaluator("scheme", "elk", "scm", 0,
+					  elk_setup, elk_evaluator_init,
+					  params_handler, getopt_options,
+					  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBERL_INTERFACE
-  evaluator_erlang = muesli_define_evaluator("erlang", "er", 0,
+  evaluator_erlang = muesli_define_evaluator("erlang", "erlang", "er", 0,
 					     erlang_setup, erlang_evaluator_init,
 					     params_handler, getopt_options,
 					     new_app_params, new_app_data);
 #endif
+#ifdef HAVE_LIBGS
+  evaluator_gs = muesli_define_evaluator("PostScript", "gs", "ps", 0,
+					 gs_setup, gs_evaluator_init,
+					 params_handler, getopt_options,
+					 new_app_params, new_app_data);
+#endif
 #ifdef HAVE_LIBGUILE
-  evaluator_guile = muesli_define_evaluator("guile", "scm", 0,
+  evaluator_guile = muesli_define_evaluator("scheme", "guile", "scm", 0,
 					    guile_setup, guile_evaluator_init,
 					    params_handler, getopt_options,
 					    new_app_params, new_app_data);
 #endif
+#ifdef HAVE_LIBKSI
+  evaluator_ksi = muesli_define_evaluator("scheme", "ksi", "scm", 0,
+					  ksi_setup, ksi_evaluator_init,
+					  params_handler, getopt_options,
+					  new_app_params, new_app_data);
+#endif
+#ifdef HAVE_LIBMATHEVAL
+  evaluator_matheval = muesli_define_evaluator("infix", "matheval", "math", 0,
+					       matheval_setup, matheval_evaluator_init,
+					       params_handler, getopt_options,
+					       new_app_params, new_app_data);
+#endif
 #ifdef HAVE_LIBMOZJS
-  evaluator_javascript = muesli_define_evaluator("javascript", "js", 0,
+  evaluator_javascript = muesli_define_evaluator("javascript", "SpiderMonkey", "js", 0,
 						 javascript_setup, javascript_evaluator_init,
 						 params_handler, getopt_options,
 						 new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBJVM
-  evaluator_jvm = muesli_define_evaluator("jvm", "class", 1,
+  evaluator_jvm = muesli_define_evaluator("jvm", "jvm", "class", 1,
 					  jvm_setup, jvm_evaluator_init,
 					  params_handler, getopt_options,
 					  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBLUA
-  evaluator_lua = muesli_define_evaluator("lua", "lua", 0,
+  evaluator_lua = muesli_define_evaluator("lua", NULL, "lua", 0,
 					  lua_setup, lua_evaluator_init,
 					  params_handler, getopt_options,
 					  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBOCTINTERP
-  evaluator_octave = muesli_define_evaluator("octave", "oct", 0,
+  evaluator_octave = muesli_define_evaluator("octave", NULL, "oct", 0,
 					     octave_setup, octave_evaluator_init,
 					     params_handler, getopt_options,
 					     new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBPARROT
-  evaluator_parrot = muesli_define_evaluator("parrot", "peg", 1,
+  evaluator_parrot = muesli_define_evaluator("parrot", NULL, "peg", 1,
 					     parrot_setup, parrot_evaluator_init,
 					     params_handler, getopt_options,
 					     new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBPERL
-  evaluator_perl = muesli_define_evaluator("perl", "pl", 0,
+  evaluator_perl = muesli_define_evaluator("perl", NULL, "pl", 0,
 					   perl_setup, perl_evaluator_init,
 					   params_handler, getopt_options,
 					   new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBPL
-  evaluator_prolog = muesli_define_evaluator("prolog", ".pl", 0,
+  evaluator_prolog = muesli_define_evaluator("prolog", "SWI Prolog", ".pl", 0,
 					     prolog_setup, prolog_evaluator_init,
 					     params_handler, getopt_options,
 					     new_app_params, new_app_data);
 #endif
 #if defined(HAVE_LIBPYTHON) && defined(HAVE_PYTHON2_6_PYTHON_H)
-  evaluator_python = muesli_define_evaluator("python", "python", 0,
+  evaluator_python = muesli_define_evaluator("python", "CPython", "py", 0,
 					     python_setup, python_evaluator_init,
 					     params_handler, getopt_options,
 					     new_app_params, new_app_data);
 #endif
+#ifdef HAVE_LIBR
+  evaluator_R = muesli_define_evaluator("R", NULL, "r", 0,
+					R_setup, R_evaluator_init,
+					params_handler, getopt_options,
+					new_app_params, new_app_data);
+#endif
 #ifdef HAVE_LIBRUBY_STATIC
-  evaluator_ruby = muesli_define_evaluator("ruby", "rb", 0,
+  evaluator_ruby = muesli_define_evaluator("ruby", NULL, "rb", 0,
 					   ruby_setup, ruby_evaluator_init,
 					   params_handler, getopt_options,
 					   new_app_params, new_app_data);
 #endif
 #if defined(HAVE_LIBSIOD) || defined(HAVE_LIBULSIOD)
-  evaluator_siod = muesli_define_evaluator("siod", "scm", 0,
+  evaluator_siod = muesli_define_evaluator("scheme", "siod", "scm", 0,
 					   siod_setup, siod_evaluator_init,
 					   params_handler, getopt_options,
 					   new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBSLANG
-  evaluator_slang = muesli_define_evaluator("s-lang", "sl", 0,
+  evaluator_slang = muesli_define_evaluator("s-lang", NULL, "sl", 0,
 					    slang_setup, slang_evaluator_init,
 					    params_handler, getopt_options,
 					    new_app_params, new_app_data);
 #endif
+#ifdef HAVE_LIBSYX
+  evaluator_smalltalk = muesli_define_evaluator("smalltalk", "syx", "st", 0,
+						syx_setup, syx_evaluator_init,
+						params_handler, getopt_options,
+						new_app_params, new_app_data);
+#endif
 #ifdef HAVE_LIBTCC
-  evaluator_tcc = muesli_define_evaluator("tcc", "c", 0,
+  evaluator_tcc = muesli_define_evaluator("Tiny C", "tcc", "c", 0,
 					  tcc_setup, tcc_evaluator_init,
 					  params_handler, getopt_options,
 					  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBTCL
-  evaluator_tcl = muesli_define_evaluator("tcl", "tcl", 0,
+  evaluator_tcl = muesli_define_evaluator("tcl", NULL, "tcl", 0,
 					  tcl_setup, tcl_evaluator_init,
 					  params_handler, getopt_options,
 					  new_app_params, new_app_data);
 #endif
-  evaluator_machine = muesli_define_evaluator("machine", "o", 1,
+  evaluator_machine = muesli_define_evaluator("machine", NULL, "o", 1,
 					      machine_code_setup, machine_code_evaluator_init,
 					      params_handler, getopt_options,
 					      new_app_params, new_app_data);
-  evaluator_stack_code = muesli_define_evaluator("stack-code", "sc", 1,
+  evaluator_stack_code = muesli_define_evaluator("stack-code", NULL, "sc", 1,
 						 stack_code_setup, stack_bytecode_evaluator_init,
 						 params_handler, getopt_options,
 						 new_app_params, new_app_data);
   {
+    /* The Lisp evaluator is configured dynamically, rather than
+       through autoconf; we look around to see what's available. */
     const char *common_lisp = getenv((char*)"COMMON_LISP");
+    const char *executable = NULL;
     if (common_lisp == NULL) {
+      /* todo: look for different Lisps on the path */
       common_lisp = (char*)"sbcl";
     }
-    evaluator_common_lisp = muesli_define_variant_evaluator(evaluator_external_pipe,
-							    "lisp",
-							    common_lisp,
-							    "information.\n* ",
-							    "* ",
-							    "(quit)\n");
+    if (strcmp(common_lisp, "sbcl") == 0) {
+      if ((executable = muesli_find_executable("sbcl")) != NULL)  {
+	evaluator_common_lisp = muesli_define_variant_evaluator(evaluator_external_pipe,
+								"Common Lisp",
+								"sbcl",
+								executable,
+								"information.\n* ",
+								"* ",
+								"(quit)\n");
+      }
+    }
+#if 0
+    /* todo: other CL implementations: clisp, cmucl, gcl, ecl; note
+       ecl can probably have a full interface of its own */
+    else if (strcmp(common_lisp, "clisp") == 0) {
+    } else if (strcmp(common_lisp, "cmucl") == 0) {
+    } else if (strcmp(common_lisp, "gcl") == 0) {
+    } else if (strcmp(common_lisp, "ecl") == 0) {
+    }
+#endif
+    if (evaluator_common_lisp != NULL) {
+      evaluator_common_lisp->array_separator = " "; /* todo: check array open and close */
+    }
   }
   {
+    /* The Haskell evaluator is configured dynamically, rather than
+       through autoconf; we look around to see what's available. */
     const char *haskell = getenv((char*)"HASKELL");
     if (haskell == NULL) {
+      /* todo: look on the path to see which known haskells are present */
       haskell = (char*)"hugs";
     }
+    /* todo: set these parameters for the specific implementation found */
     evaluator_haskell = muesli_define_variant_evaluator(evaluator_external_pipe,
 							"haskell",
+							"hugs",
 							haskell,
 							"Type :? for help\nHugs> ",
 							"Hugs> ",
 							":quit\n");
+    /* todo: what is Haskell's array syntax? */
   }
 }
 
@@ -692,9 +812,9 @@ muesli_evaluator_repl(const char *evaluator_name)
     fprintf(stderr, "Could not find an evaluator called %s\n", evaluator_name);
   } else if (OUR_OWN_REPL || evaluator->repl == NULL) {
     int running = 1;
-    char *prompt = (char*)malloc(strlen(evaluator->name) + 2);
-    sprintf(prompt, "%s>", evaluator->name);
-    fprintf(stderr, "Using makeshift Read-Eval-Print loop for %s\n", evaluator->name);
+    char *prompt = (char*)malloc(strlen(evaluator->language_name) + 2);
+    sprintf(prompt, "%s>", evaluator->language_name);
+    fprintf(stderr, "Using makeshift Read-Eval-Print loop for %s\n", evaluator->implementation_name);
     fprintf(stderr, "Use quit to go back to application run or exit to leave application\n");
     do {
       char *input;
@@ -739,7 +859,7 @@ muesli_evaluator_repl(const char *evaluator_name)
     } while (running);
     free(prompt);
   } else {
-    printf("Calling Read-Eval-Print loop of %s\n", evaluator->name);
+    printf("Calling Read-Eval-Print loop of %s\n", evaluator->implementation_name);
     evaluator->repl(evaluator);
   }
 }
@@ -887,23 +1007,23 @@ muesli_find_executable(const char *basename)
 	strcat(prog_name, "/");
       }
       strcat(prog_name, basename);
-      fprintf(stderr, "Trying \"%s\"\n", prog_name);
+      // fprintf(stderr, "Trying \"%s\"\n", prog_name);
       if (stat(prog_name, &statbuf) == 0) {
-	fprintf(stderr, "Statted %s\n", prog_name);
+	// fprintf(stderr, "Statted %s\n", prog_name);
 	if (statbuf.st_mode & S_IXOTH) {
-	  fprintf(stderr, "OK by world\n");
+	  // fprintf(stderr, "OK by world\n");
 	  break;
 	}
 	if ((statbuf.st_mode & S_IXUSR)
 	    && ((statbuf.st_uid == getuid())
 		|| (statbuf.st_uid == geteuid())))
 	  {
-	    fprintf(stderr, "OK by user\n");
+	    // fprintf(stderr, "OK by user\n");
 	    break;
 	  }
 	if (statbuf.st_mode & S_IXGRP) {
 	  if (statbuf.st_gid == getgid()) {
-	    fprintf(stderr, "OK by main group\n");
+	    // fprintf(stderr, "OK by main group\n");
 	    break;
 	  } else {
 	    gid_t gids[NGROUPS_MAX];
@@ -1009,9 +1129,26 @@ muesli_get_parameter(evaluator_interface *interface,
 char *
 muesli_malloc_copy_string(const char *original)
 {
-  char *copy = (char*)malloc(strlen(original));
-  strcpy(copy, original);
-  return copy;
+  if (original == NULL) {
+    return NULL;
+  } else {
+    char *copy = (char*)malloc(strlen(original)+1);
+    strcpy(copy, original);
+    return copy;
+  }
+}
+
+char *
+muesli_malloc_copy_string_n(const char *original, unsigned int length)
+{
+  if (original == NULL) {
+    return NULL;
+  } else {
+    char *copy = (char*)malloc(length + 1);
+    strncpy(copy, original, length);
+    copy[length] = '\0';		/* should we do this? */
+    return copy;
+  }
 }
 
 #endif
